@@ -17,12 +17,13 @@ The detection module provides classes representing Panther detections: Rules, Po
 from .. import _utilities
 
 __all__ = [
-    "DynamicField",
+    "DynamicStringField",
+    "DynamicDestinations",
+    "AlertGrouping",
     "_BaseFilter",
     "PythonFilter",
     "_BaseUnitTest",
     "JSONUnitTest",
-    "DictUnitTest",
     "Rule",
     "SeverityLow",
     "SeverityInfo",
@@ -42,7 +43,7 @@ ReportKeyMITRE = "MITRE ATT&CK"
 
 
 @dataclasses.dataclass(frozen=True)
-class DynamicField(_utilities.ConfigNode):
+class DynamicStringField(_utilities.ConfigNode):
     """Make a field dynamic based on the detection input
 
     Attributes:
@@ -52,14 +53,14 @@ class DynamicField(_utilities.ConfigNode):
     """
 
     # required
-    func: typing.Callable[[typing.Any], bool]
+    func: typing.Callable[[typing.Any], str]
 
     # optional
     fallback: str = ""
 
     # internal private methods
     def _typename(self) -> str:
-        return "DynamicField"
+        return "DynamicStringField"
 
     def _output_key(self) -> str:
         return ""
@@ -68,6 +69,60 @@ class DynamicField(_utilities.ConfigNode):
         return ["func", "fallback"]
 
 
+@dataclasses.dataclass(frozen=True)
+class DynamicDestinations(_utilities.ConfigNode):
+    """Make destinations dynamic based on the detection input
+
+    Attributes:
+        - func -- Dynamic handler (required)
+        - fallback -- Fallback value in case the dynamic handler fails (optional, default: None)
+
+    """
+
+    # required
+    func: typing.Callable[[typing.Any], typing.List[str]]
+
+    # optional
+    fallback: typing.Optional[typing.List[str]] = None
+
+    # internal private methods
+    def _typename(self) -> str:
+        return "DynamicDestinations"
+
+    def _output_key(self) -> str:
+        return ""
+
+    def _fields(self) -> typing.List[str]:
+        return ["func", "fallback"]
+
+
+@dataclasses.dataclass(frozen=True)
+class AlertGrouping(_utilities.ConfigNode):
+    """Configuration for how an alert is grouped
+
+    Attributes:
+        - group_by -- Function to generate a key for grouping matches (optional, default: None)
+        - period_minutes -- How long should matches be grouped into an alert after the first match (optional, default: 15)
+
+    """
+
+    # optional
+    group_by: typing.Optional[typing.Callable[[typing.Any], str]] = None
+
+    # optional
+    period_minutes: int = 15
+
+    # internal private methods
+    def _typename(self) -> str:
+        return "AlertGrouping"
+
+    def _output_key(self) -> str:
+        return ""
+
+    def _fields(self) -> typing.List[str]:
+        return ["group_by", "period_minutes"]
+
+
 @dataclasses.dataclass(frozen=True)
 class _BaseFilter(_utilities.ConfigNode):
     """Base filter"""
@@ -96,7 +151,7 @@ class PythonFilter(_BaseFilter):
     # required
     func: typing.Callable[
         [
-            typing.Any,
+            typing.Optional[typing.Any],
             typing.Optional[typing.Dict[str, typing.Union[str, int, float, bool]]],
         ],
         bool,
@@ -139,6 +194,7 @@ class JSONUnitTest(_BaseUnitTest):
 
     Attributes:
         - data -- json data (required)
+        - expect_match -- should this event trigger the rule? (required)
         - name -- name of the unit test (required)
 
     """
@@ -146,6 +202,9 @@ class JSONUnitTest(_BaseUnitTest):
     # required
     data: str
 
+    # required
+    expect_match: bool
+
     # required
     name: str
 
@@ -157,30 +216,7 @@ class JSONUnitTest(_BaseUnitTest):
         return ""
 
     def _fields(self) -> typing.List[str]:
-        return ["data", "name"]
-
-
-@dataclasses.dataclass(frozen=True)
-class DictUnitTest(_BaseUnitTest):
-    """Unit test with python dict content
-
-    Attributes:
-        - data -- json data (required)
-
-    """
-
-    # required
-    data: typing.Dict[str, typing.Any]
-
-    # internal private methods
-    def _typename(self) -> str:
-        return "DictUnitTest"
-
-    def _output_key(self) -> str:
-        return ""
-
-    def _fields(self) -> typing.List[str]:
-        return ["data"]
+        return ["data", "expect_match", "name"]
 
 
 @dataclasses.dataclass(frozen=True)
@@ -193,13 +229,19 @@ class Rule(_utilities.ConfigNode):
         - rule_id -- ID for the rule (required)
         - severity -- Severity for the rule (required)
         - alert_context -- Optional JSON to attach to alerts generated by this rule (optional, default: None)
+        - alert_grouping -- Configuration for how an alert is grouped (optional, default: None)
+        - alert_title -- Title to use in the alert (optional, default: None)
+        - description -- Description for the rule (optional, default: "")
+        - destinations -- Alert destinations for the rule (optional, default: None)
         - enabled -- Short description for the query (optional, default: True)
+        - group -- Number of matches received before an alert is triggered (optional, default: 1)
         - name -- Display name for the rule (optional, default: "")
         - reference -- Reference for the rule (optional, default: "")
         - reports -- Report mappings for the rule (optional, default: None)
         - runbook -- Runbook for the rule (optional, default: "")
         - summary_attrs -- Summary Attributes for the rule (optional, default: None)
         - tags -- Tags for the rule (optional, default: None)
+        - threshold -- Number of matches received before an alert is triggered (optional, default: 1)
         - unit_tests -- Define event filters for the rule (optional, default: None)
 
     """
@@ -214,25 +256,44 @@ class Rule(_utilities.ConfigNode):
     rule_id: str
 
     # required
-    severity: str
+    severity: typing.Union[str, DynamicStringField]
 
     # optional
-    alert_context: typing.Optional[typing.Callable[[typing.Any], str]] = None
+    alert_context: typing.Optional[
+        typing.Callable[[typing.Any], typing.Dict[str, typing.Any]]
+    ] = None
+
+    # optional
+    alert_grouping: typing.Optional[AlertGrouping] = None
+
+    # optional
+    alert_title: typing.Optional[typing.Callable[[typing.Any], str]] = None
+
+    # optional
+    description: typing.Optional[typing.Union[str, DynamicStringField]] = ""
+
+    # optional
+    destinations: typing.Optional[
+        typing.Union[str, typing.List[str], DynamicDestinations]
+    ] = None
 
     # optional
     enabled: bool = True
 
     # optional
-    name: typing.Optional[typing.Union[str, DynamicField]] = ""
+    group: int = 1
 
     # optional
-    reference: typing.Optional[typing.Union[str, DynamicField]] = ""
+    name: typing.Optional[str] = ""
+
+    # optional
+    reference: typing.Optional[typing.Union[str, DynamicStringField]] = ""
 
     # optional
     reports: typing.Optional[typing.Dict[str, typing.List[str]]] = None
 
     # optional
-    runbook: typing.Optional[typing.Union[str, DynamicField]] = ""
+    runbook: typing.Optional[typing.Union[str, DynamicStringField]] = ""
 
     # optional
     summary_attrs: typing.Optional[typing.List[str]] = None
@@ -240,9 +301,12 @@ class Rule(_utilities.ConfigNode):
     # optional
     tags: typing.Optional[typing.Union[str, typing.List[str]]] = None
 
+    # optional
+    threshold: int = 1
+
     # optional
     unit_tests: typing.Optional[
-        typing.Union[_BaseFilter, typing.List[_BaseFilter]]
+        typing.Union[_BaseUnitTest, typing.List[_BaseUnitTest]]
     ] = None
 
     # internal private methods
@@ -259,12 +323,18 @@ class Rule(_utilities.ConfigNode):
             "rule_id",
             "severity",
             "alert_context",
+            "alert_grouping",
+            "alert_title",
+            "description",
+            "destinations",
             "enabled",
+            "group",
             "name",
             "reference",
             "reports",
             "runbook",
             "summary_attrs",
             "tags",
+            "threshold",
             "unit_tests",
         ]
